; Assembly Listing of the ZX Spectrum program UDG Editor
; 
; CPU Type: Z80
; 
; by Geoffrey Wearmouth
;
; 1985
; -----------------------------------------------------------------------------

#define DEFB .BYTE      
#define ORG  .ORG
#define EQU  .EQU

; -----------------
; ROM routines used
; -----------------
;
CL_ALL 		EQU	$0DAF		; clear display file
CL_SET		EQU	$0DD9		; Sets print position from column/line
BEEPER		EQU 	$03B5		; produce keyboard click
INDEXER		EQU	$16DC		; Index look-up table of paired values.
L0AE8		EQU	$0AE8   	; PO-STORE (upper) Store print position
L0AEC		EQU	$0AEC		; PO-STORE
PLOT_SUB	EQU	$22E5 		; Plot BC
L1715		EQU	$1715		; location holding JP (HL)
 

; also PRINT-A via RST 10h		; Print Accumulator


; ---------------------
; System Variables used
; ---------------------
;
LAST_K	EQU	23560			; $5C08	also (iy-32h) Stores newly 
					; pressed key (from interrupt)
PIP	EQU	23609			; $5C39 also (iy+01h) length of keyboard
					; click
UDG	EQU	23675			; $5C7B Address of first user-defined 
					; graphic.
FLAGS2 	EQU 	23658			; $5C6A also (iy+30h) controls CAPSLOCK
P-FLAG	EQU	23697			; $5C91 also (iy+57h) controls OVER/INV


ORG  	5b00h

; ----------
; INITIALIZE
; ----------

        nop     			; spare
        call    CL_ALL			; ROM CL-ALL clears whole display $0DAF
        ld      (iy+30h),a		; set FLAGS2 to zero CAPSLOCK off.

; ---------------------------------------------------
; Draw grid outline east and south like a flipped 'L'
; ---------------------------------------------------
;

        ld      a,80h			; Set leftmost pixel
        ld      hl,47f4h		; set screen starting point
        ld      e,20h			; set DE to thirty two (D is zero)
        ld      b,41h			; set pixel count to 65

loop65: add     hl,de			; add thirty two
        ld      (hl),a			; Poke with 128
        djnz    loop65			; repeat 65 times drawing a vertical 
					; line

loop8:  dec     l			; decrement screen address moving left
        dec     (hl)			; change zero to 255
        rrca    			; check the eight count
        jr      nc,loop8		; back until eight bytes filled.

; ---------------------------------------
; print line of ASCII chars A-U once only
; ---------------------------------------
;

        ld      e,40h           	; start chr "A" (65) - 1
        ld      hl,4025h        	; print position address

        call    set21   		; print 21 chars starting chr 65 A

        xor     a               	; set initial udg to zero
        nop     			; spare

; --------------------------------
; print line of UDGs at every save
; --------------------------------
;

toplp:  ld      (poke1+1),a       	; set current udg 

        call    getch			; routine get character prints lower 
					; cell character.

        ld      e,8fh           	; set start to udg-A - 1
        ld      hl,4065h		; set screen position address

        call    set21			; print 21 chars starting at udg-A

; ------------------------------------------------------------------
; Now highlight current character by setting colour attribute bright
; ------------------------------------------------------------------
; 

        ld      hl,5865h        	; attribute file pos of start of UDG set

poke1:  ld      de,0000h		; Modified value of UDG number.
        add     hl,de           	; offset into attribute file.
        ld      (hl),78h		; set attribute to bright black on white


; ----------------------------------------------------------------
; The mid loop is executed whenever the grid or cursor has changed
; ----------------------------------------------------------------
;

midlp:  ld      hl,578fh        	; screen position of udg bottom byte.
        ld      e,14h			; starting 'x' column position of grid.
        ld      bc,0101h       		; initialize two 8 counters.
        ld      (iy+57h),b  		; set P-FLAG to 'OVER 1'.

gridlp: ld      a,e			; reduce grid position by 
        sub     28h			; 32 + 8 
        ld      e,a			; bytes.

bytelp: ld      d,50h			; starting position for grid address hi
        rlc     (hl)			; rotate byte of udg
        sbc     a,a			; Form 255 if carry else zero
        or      80h			; set leftmost pixel to make grid.

chrlp:  dec     d			; next gridline up
        ld      (de),a			; place 128 or 25. 
        rlc     c			; Rotate the 8 counter.
        jr      nc,chrlp       		; Repeat for 8 vertical screen bytes

        sbc     a,a			; Always 255 - horizontal bar of grid
        ld      (de),a			; Overwrite last location with $FF
        inc     e			; Next byte to right.
        rlc     b			; Rotate bit counter.
        jr      nc,bytelp        	; Back if more to do.

        dec     h			; Decrement cell address
        bit     4,h			; Test - still in lower third ?
        jr      nz,gridlp   		; Back if so

; ----------------------------------------------------------------------------
; now print cross cursor (+) using ROM print routine for xor printing (OVER 1)
; The cursor plot coordinates are converted to print coordinates
; ----------------------------------------------------------------------------
;

poke2:  ld      bc,0f78h		; cursor plot pos at pixel level 
					; y = $08-$0f from bottom
					; x = $78-$7f from left
					; Note. the above 2 values are modified
					; rather than using a separate variable.

        push    bc			; *Save plot values for later.
        inc     b			; make y range $09 - $10 at character
					; level from bottom.

        ld      a,8dh			; make x 
        sub     c			; range
        ld      c,a			; $15 - $0E (from right)

        call    CL_SET			; ROM CL-SET sets print position $0DD9

        ld      e,(iy-01h)		; Fetch value of PIP system variable.
        ld      hl,00c8h		; Set tone value.
        call    BEEPER			; ROM BEEPER gives a click $03B5.

        ld      a,2bh			; Prepare "+" character.
        rst     10h			; ROM print-a.

; ----------------------------------------------------------------------
; main loop waits first for a keypress updated by the interrupt routine.
; ----------------------------------------------------------------------

        ld      (5c08h),a		; Set system variable LAST-K to zero.

loopk:  ld      c,(iy-32h)		; Get key code from LAST-K.

        ld      hl,table		; look up table.

        call    INDEXER			; Call ROM routine INDEXER $16dc

        jr      nc,loopk		; Back with no matching key.

        ld      l,(hl)			; else convert to address of routine.

        pop     bc			;* Restore position.
        ld      a,(poke1+1)		; Fetch UDG number 0-20

        call    L1715			; Call (hl)

        jr      z,midlp			; back to grid if zero

        jr      toplp			; else print udg set as well

; --------------------------
; print set of 21 characters
; --------------------------

set21:  ld      c,h			; make sure C > 21 by using 64
        call    L0AE8			; ROM set print position df-cc from HL

        ld      b,15h                   ; set count to 21

loop21: inc     e			; increment character
        ld      a,e			; transfer to accumulater
        rst     10h			; use ROM routine PRINT-A
        djnz    loop21			; repeat for all 21

        ret     			; return

; -----------------------------------------------------------------------
; quit - drop the return address and continue into next to save character
; -----------------------------------------------------------------------

; quit
;
quit:   pop     hl			; drop return address and continue
					; saving character.

; -----------------------------------------------------------
; next - move to next character but save current design first
; -----------------------------------------------------------
;
next:   ld      hl,(UDG)		; System Variable UDG $5C7B
        push    af			; save udg pointer 0-20.
        rlca    			; multiply
        rlca    			; by
        rlca    			; eight
        ld      e,a			; result to e	
        add     hl,de			; add (d=0)
        ld      de,508fh		; Make DE point to top of cell

        ld      b,08h			; eight bytes to transfer.

savelp: ld      a,(de)			; fetch a byte from cell
        ld      (hl),a			; transfer to memory
        inc     d			; move down cell.
        inc     hl			; next UDG memory location
        djnz    savelp 			; repeat for all 8 bytes

        pop     af			; bring back udg pointer (0-20)
        inc     a			; increment
        cp      15h			; compare to 21
        ret     			; return (with zero set if at limit)

; ------------------------------------------------------------
; back - Call next and backtrack. Backtrack again if possible.
; ------------------------------------------------------------
;

back:   call    next                   	; call next
        dec     a			; decrement udg pointer
        ret     z			; return if at start

        dec     a			; decrement udg pointer
        inc     e			; reset zero flag
        ret  				; return signalling new character

; ------------------------   
; down - twice and then up
; ------------------------   

down:   dec     b			; down
        dec     b			; and again to counter next

up:  	inc     b			; up
        bit     3,b			; still in range ?
        jr      test			; forward to common test

left:  	dec     c			; left
        dec     c			; and again to counter next

right:  inc     c			; right

    	bit     3,c			; test in range

test:   ret     z			; return if out of range

        ld      (poke2+1),bc		; else update new position
        xor     a			; set zero to redraw grid
        ret     			; return 

; -------------------------------------------------------
; space character - toggle pixel by XOR plotting - OVER 1 
; -------------------------------------------------------
;

space:
        jp      PLOT_SUB		; ROM PLOT-SUB - Plot BC using OVER 1

; --------------------------------------------------------
; clear character - by printing solid character in inverse
; --------------------------------------------------------
;
clear:  ld      d,$03			; will become $04 INVERSE 1

; -------------------------------------------------------------
; inverse - invert the pixels by XOR printing the solid graphic
; -------------------------------------------------------------
;
inv:  	sbc     a,a			; make character $ff -
					; will become CHR$ 143 WHEN $90 added.
        inc     d			; set register d to $01 or $04

; ------------------------------------------------
; get character - print current UDG in lower cell
; ------------------------------------------------
;
getch:  add     a,90h			; convert 0-20 to real UDG chr or 
					; set $FF to $8F (solid graphic - 143)
        ld      (iy+57h),d		; Set P-FLAG 
        ld      hl,508fh		; Set location of cell

        call    L0AEC			; ROM routine PO-STORE (upper screen)

        rst     10h                     ; ROM print-a
        ret     			; return

; -------------
; LOOK UP TABLE
; -------------

table:  
	DEFB	"q", quit  % $5B00	; $71, $a0 quit
	DEFB	"i", inv   % $5B00	; $69, $da inverse
	DEFB	"n", next  % $5B00	; $6e, $a1 next
	DEFB  	"c", clear % $5B00	; $63, $d8 clear
	DEFB	"g", getch % $5B00	; $67, $dc get
	DEFB	"b", back  % $5B00	; $62, $ba back
	DEFB	" ", space % $5B00	; $20, $d5 toggle
	DEFB	$08, left  % $5B00	; $08, $c9 left
	DEFB	$09, right % $5B00	; $09, $cb right
	DEFB	$0a, down  % $5B00	; $0a, $c2 down
	DEFB	$0b, up    % $5B00	; $0b, $c4 up
	DEFB	$00			; zero end-marker 


.end     
        
; -------------------------------------------------
; Save code with SAVE 'u0' CODE 23296,256
; To use load the code and use RANDOMIZE USR 23296
; It is deleted by LPRINT, LLIST and COPY
; -------------------------------------------------
